# Generated by Django 2.0.2 on 2018-02-25 05:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0001_initial'),
        ('crmify', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='lead',
            name='anchor',
            field=models.OneToOneField(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='crm_lead', to='sales.UserProfile'),
            preserve_default=False,
        ),
    ]
