# Generated by Django 2.0.2 on 2018-02-25 02:37

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Lead',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lead_source', models.CharField(blank=True, default=None, max_length=100, null=True)),
                ('first_name', models.CharField(max_length=150)),
                ('last_name', models.CharField(max_length=150)),
                ('email', models.CharField(blank=True, default=None, max_length=200, null=True)),
                ('secondary_email', models.CharField(blank=True, default=None, max_length=200, null=True)),
                ('phone', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('mobile_phone', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('dob', models.DateField(blank=True, default=None, max_length=50, null=True)),
                ('street_address', models.CharField(blank=True, default=None, max_length=200, null=True)),
                ('city', models.CharField(blank=True, default=None, max_length=200, null=True)),
                ('state', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('country', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('external_id', models.CharField(max_length=200, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
